from typing import Any, Dict, Tuple

from aiohttp import ClientResponse, ClientSession
from httpx import AsyncClient


class UrlHelpers:
    """
    Class to help with the creation and handling of urls
    """

    @staticmethod
    async def shorten_url(
        long_url: str,
    ) -> str:
        """
        Shorten a long URL using the tinyurl.com API.

        Args:
            long_url: The long URL to shorten.

        Returns:
            The shortened URL.
        """
        api_url = "https://tinyurl.com/api-create.php?url=" + long_url
        async with AsyncClient() as client:
            r = await client.get(api_url)
        return r.text


class AioHttp:
    """
    Class to help with the creation and handling of aiohttp requests.
    """

    @staticmethod
    async def get_json(
        link: str,
        **kwargs,
    ) -> Tuple[Dict[str, Any], ClientResponse]:
        """
        Get the JSON response from a link.

        Args:
            link: The URL to send request to.

        Returns:
            The JSON response from the link.
            The response object.
        """
        async with ClientSession() as session:
            async with session.get(link, **kwargs) as resp:
                return await resp.json(), resp

    @staticmethod
    async def get_text(
        link: str,
        **kwargs,
    ) -> Tuple[str, ClientResponse]:
        """
        Get the text response from a link.

        Args:
            link: The URL to send request to.

        Returns:
            The TEXT response from the link.
            The response object.
        """
        async with ClientSession() as session:
            async with session.get(link, **kwargs) as resp:
                return await resp.text(), resp

    @staticmethod
    async def get_raw(
        link: str,
        **kwargs,
    ) -> Tuple[bytes, ClientResponse]:
        """
        Get the raw response from a link.

        Args:
            link: The URL to send request to.

        Returns:
            The RAW response from the link.
            The response object.
        """
        async with ClientSession() as session:
            async with session.get(link, **kwargs) as resp:
                return await resp.read(), resp

    @staticmethod
    async def post_json(
        link: str,
        data: Dict[str, Any],
        **kwargs,
    ) -> Tuple[Dict[str, Any], ClientResponse]:
        """
        Post the JSON data to a link.

        Args:
            link: The URL to post request to.

        Returns:
            The JSON response from the link.
            The response object.
        """
        async with ClientSession() as session:
            async with session.post(link, json=data, **kwargs) as resp:
                return await resp.json(), resp



from functools import reduce
from types import SimpleNamespace
from typing import Any, Dict, List, Tuple


class Namespace(SimpleNamespace):
    """
    A nested namespace class that allows for nested dictionaries to be converted to a nested namespace.
    """

    def __init__(
        self,
        dictionary: Dict[str, Any] = None,
        **kwargs,
    ) -> None:
        """
        Initialize the namespace with a dictionary.

        Args:
            dictionary: The dictionary to initialize the namespace with.
            kwargs: Any additional keyword arguments to pass to the namespace.
        """
        if dictionary is None:
            dictionary = {}
        super().__init__(**kwargs)
        for key, value in dictionary.items():
            if isinstance(value, list):
                self.__dict__[key] = [Namespace(item) for item in value]
            elif isinstance(value, dict):
                self.__setattr__(key, Namespace(value))
            else:
                self.__setattr__(key, value)

    def to_dict(self) -> Dict[str, Any]:
        """
        Convert the namespace to a dictionary.

        Returns:
            The namespace as a dictionary.
        """
        for key, value in self.__dict__.items():
            if isinstance(value, Namespace):
                self.__dict__[key] = value.to_dict()
        return self.__dict__

    def keys(self) -> List[str]:
        """
        Get all top level keys of dictionary.

        Returns:
            The top level keys of the namespace.
        """
        return list(self.__dict__.keys())

    def items(self) -> Tuple[str, Any]:
        """
        Get all items of dictionary.

        Returns:
            The items of the namespace.
        """
        return self.__dict__.items()

    def rsetattr(self, attr: str, val: Any) -> None:
        """
        Set nested attribute of a nested namespace.

        Args:
            attr: The attribute to set.
            val: The value to set the attribute to.
        """
        pre, _, post = attr.rpartition(".")
        return setattr(self.rgetattr(pre) if pre else self, post, val)

    def rgetattr(self, attr: str, *args) -> Any:
        """
        Get nested attribute of a nested namespace.

        Args:
            attr: The attribute to get.
            args: Any additional arguments to pass to the getattr function.

        Returns:
            The value of the attribute.
        """

        def _getattr(obj: dict, attr: str) -> Any:
            return getattr(Namespace(obj), attr, *args)

        return reduce(_getattr, [self] + attr.split("."))

    def __iter__(self) -> List[Any]:
        """
        Enables iteration in class.

        Return:
            The items of the namespace.
        """
        return iter(self.__dict__)



from hachoir.metadata import extractMetadata
from hachoir.parser import createParser


class VideoTools:
    @staticmethod
    def extract_vid_data(output_vid: str):
        metadata = extractMetadata(createParser(output_vid))
        duration = metadata.get("duration").seconds if metadata.has("duration") else 0
        width = metadata.get("width") if metadata.has("width") else 100
        height = metadata.get("height") if metadata.has("height") else 100
        return duration, width, height



from typing import Any, Dict, List, Optional, Tuple, Union

from motor.motor_asyncio import AsyncIOMotorClient


class AsyncMongoDB:
    """
    Async Class for interacting with Bot database.
    """

    def __init__(
        self,
        database_url: str,
        database_name: Optional[str] = "test",
        collection: Optional[str] = "test",
    ) -> None:
        """
        Initialize the database.

        Args:
            database_url: The url of the database.
            database_name: The name of the database, defaults to "test".
            collection: The collection to use, defaults to "test".
        """
        self.db_client = AsyncIOMotorClient(database_url)
        self.main_db = self.db_client[database_name]
        self.collection = self.main_db[collection]

    async def insert_one(
        self,
        document: Dict[str, Any],
    ) -> Optional[str]:
        """
        Insert one document into collection.

        Args:
            document: The document to insert.

        Returns:
            The inserted_id of the inserted document.
        """
        result = await self.collection.insert_one(document)
        return repr(result.inserted_id)

    async def find_one(
        self,
        query: Dict[str, Any],
    ) -> Union[Dict[str, Any], None]:
        """
        Find one entry from collection.

        Args:
            query: The query to find the entry.

        Returns:
            The entry that matches the query.
        """
        result = await self.collection.find_one(query)
        return result if result else None

    async def find_all(
        self,
        query: Optional[Union[Dict[str, Any], None]] = None,
    ) -> List[Dict[str, Any]]:
        """
        Find all entries from collection.

        Args:
            query: The query to find the entries.

        Returns:
            The list of entries that match the query.
        """
        if query is None:
            query = {}
        return [document async for document in self.collection.find(query)]

    async def count(self, query: Optional[Union[Dict[str, Any], None]] = None) -> int:
        """
        Count the number of entries in collection.

        Args:
            query: The query to count the entries.

        Returns:
            The number of entries that match the query.
        """
        if query is None:
            query = {}
        return await self.collection.count_documents(query)

    async def delete_one(
        self,
        query: Optional[Union[Dict[str, Any], None]],
    ) -> Tuple[int, int]:
        """
        Delete one entry from collection.

        Args:
            query: The query to delete the entry.

        Returns:
            The number of entries after deleting the query.
        """
        before_delete = await self.collection.count(query)
        await self.collection.delete_many(query)
        after_delete = await self.collection.count_documents({})
        return before_delete, after_delete

    async def replace(
        self,
        query: Dict[str, Any],
        new_data: Dict[str, Any],
    ) -> Tuple[Dict[str, Any], Dict[str, Any]]:
        """
        Replace one entry from collection.

        Args:
            query: The query to replace the entry.
            new_data: The new data to replace the entry with.

        Returns:
            The old data and the new data.
        """
        old = await self.collection.find_one(query)
        _id = old["_id"]
        await self.collection.replace_one({"_id": _id}, new_data)
        new = await self.collection.find_one({"_id": _id})
        return old, new

    async def update(
        self,
        query: Dict[str, Any],
        update: Dict[str, Any],
    ) -> Tuple[Union[int, str], Dict[str, Any]]:
        """
        Update one entry from collection.

        Args:
            query: The query to update the entry.
            update: The update to update the entry with.

        Returns:
            The number of entries after updating the query and the updated document.
        """
        result = await self.collection.update_one(query, {"$set": update})
        new_document = await self.collection.find_one(query)
        return result.modified_count, new_document

    async def db_command(
        self,
        command: str,
    ) -> Optional[str]:
        """
        Execute a database command.

        Args:
            command: The command to execute.

        Returns:
            The result of the command.
        """
        result = await self.main_db.command(command)
        return result



from typing import Optional

from PIL import Image


class ImageTools:
    """
    Class for image manipulation
    """

    @staticmethod
    async def compress_image(
        image_path: str,
        quality: Optional[int] = 25,
    ) -> str:
        """
        Compress an image using Pillow.

        Args:
            image_path: The path to the image to compress.
            quality: The quality of the compressed image.

        Returns:
            The path to the compressed image.
        """
        image_name = image_path.split("/")[-1]
        new_filename = f"compressed_{image_name}"

        pic = Image.open(image_path)
        pic = pic.resize(pic.size, Image.LANCZOS)
        pic.save(
            new_filename,
            optimize=True,
            quality=quality,
        )
        return new_filename



from typing import Union


class Formatters:
    """
    Class for formatting data.
    """

    @staticmethod
    def humanbytes(
        size: Union[int, str],
    ) -> str:
        """
        Human friendly file size

        Args:
            size: The size in bytes.

        Returns:
            The size in a human friendly format.
        """
        if not size:
            return ""
        power = 2**10
        n = 0
        Dic_powerN = {0: " ", 1: "Ki", 2: "Mi", 3: "Gi", 4: "Ti"}
        while size > power:
            size /= power
            n += 1
        return str(round(size, 2)) + " " + Dic_powerN[n] + "B"

    @staticmethod
    def time_formatter(
        milliseconds: int,
    ) -> str:
        """
        Format time from milliseconds to hh:mm:ss

        Args:
            milliseconds: The time in milliseconds.

        Returns:
            The formatted time.
        """
        seconds, milliseconds = divmod(int(milliseconds), 1000)
        minutes, seconds = divmod(seconds, 60)
        hours, minutes = divmod(minutes, 60)
        days, hours = divmod(hours, 24)
        tmp = (
            ((str(days) + "d, ") if days else "")
            + ((str(hours) + "h, ") if hours else "")
            + ((str(minutes) + "m, ") if minutes else "")
            + ((str(seconds) + "s, ") if seconds else "")
            + ((str(milliseconds) + "ms, ") if milliseconds else "")
        )
        return tmp[:-2]

    @staticmethod
    def get_time_in_seconds(time: str) -> int:
        """
        Convert time in a string format to seconds.

        Args:
            time: The time in string format with m/h/d/w/m/y.

        Returns:
            The time in seconds.
        """
        times = {
            "s": 1,
            "m": 60,
            "h": 3600,
            "d": 86400,
            "w": 604800,
            "m": 2630000,
            "y": 31536000,
        }
        return int(time[:-1]) * times[time[-1]]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pypers']

package_data = \
{'': ['*']}

install_requires = \
['Pillow>=9.2.0,<10.0.0',
 'aiohttp>=3.8.1,<4.0.0',
 'dnspython>=2.2.1,<3.0.0',
 'hachoir>=3.1.3,<4.0.0',
 'httpx>=0.22,<0.24',
 'motor>=3.0.0,<4.0.0']

setup_kwargs = {
    'name': 'pypers',
    'version': '1.2.10',
    'description': 'A utilities module made for personal use, has small snippets which I use in many projects.',
    'long_description': '# pypers\n\n<p align="center">\n    <a href="https://pypi.org/project/PyPers/"><img src="https://img.shields.io/pypi/v/PyPers" alt="PyPI"></a>\n    <a href="https://github.com/Divkix/PyPers/actions"><img src="https://github.com/Divkix/PyPers/workflows/CI%20%28pip%29/badge.svg" alt="CI (pip)"></a>\n    <a href="https://pypi.org/project/pypers/"><img src="https://img.shields.io/pypi/wheel/PyPers.svg" alt="PyPI - Wheel"></a>\n    <a href="https://pypi.org/project/pypers/"><img src="https://img.shields.io/pypi/pyversions/PyPers.svg" alt="Supported Python Versions"></a>\n    <a href="https://pepy.tech/project/PyPers"><img src="https://pepy.tech/badge/PyPers" alt="Downloads"></a>\n</p>\n\nPackage with helper scripts.\n\nContains some helper function which make up its name: python+helpers = pypers.\n\nRead the docs:\n\nhttps://pypers.divkix.me\n\n\n[![Sponsor](https://www.datocms-assets.com/31049/1618983297-powered-by-vercel.svg)](https://vercel.com/?utm_source=divideprojects&utm_campaign=oss)\n',
    'author': 'Divkix',
    'author_email': 'divkix@divkix.me',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://pypers.divkix.me',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
